/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef hppDynamicObstacle_H
#define hppDynamicObstacle_H

#include <iostream>
#include <fstream>

#include "kcd2/kcdObject.h"
#include "KineoWorks2/kwsEdge.h"
#include <set>

using namespace::std;


// class ChppDynamicEnvHandler;

KIT_PREDEF_CLASS ( ChppDynamicObstacle );
KIT_PREDEF_CLASS ( ChppDynamicEnvHandler );

//#define DEBUG_DO(x)
#define DEBUG_DO(x) std::cerr << "ChppDynamicObstacle :" << x << std::endl

typedef struct point3D
{
	double x, y, z;
}
point3D;


/**
  \brief  The abstract class of ChppDynamicObstacle.
 */

class ChppDynamicObstacle
{
	public:

		virtual ~ChppDynamicObstacle();

		/** \brief Return the updated cell updateFlag  */
		virtual ktStatus computeModifiedWorkSpaceCells() = 0;

		/** \brief Set attObstacleModifiedFlag to true. */
		ktStatus obstacleIsModified ();

		/** \brief Set attObstacleModifiedFlag to false. */
		ktStatus roadmapIsModified ();

		/** \brief Return attObstacleIsModifiedFlag  */
		bool obstacleModificationFlag();

		/** \brief Return the dynamic obstacle  */
		virtual CkcdObjectShPtr getModifiedObstacle()=0;




	protected:


		/** \brief Constructor. */
		ChppDynamicObstacle ( ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief Set DynamicEnvHandler  */
		ChppDynamicEnvHandlerWkPtr getDynamicEnvHandler ();

		/** \brief initialization: store weak pointer to itself */
		ktStatus init ( const ChppDynamicObstacleWkPtr &inWeakPtr );



	private:
		 /** \brief Store weak pointer to object  */
		 ChppDynamicObstacleWkPtr attWeakPtr;

		/** \brief The flag which indicates wheather the obstacle should be used in roadmap modificition or not. */
		bool attObstacleIsModifiedFlag;

		/** \brief A weak pointer to dynamic handler to have access to the cells */
		ChppDynamicEnvHandlerWkPtr attDynamicEnvHandler;

		/**
		   \if 0
		   \brief Count the number of object created and not destroyed
		*/
		static unsigned int nbObject;
};
#endif
