/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#include <iostream>
#include <fstream>


#ifndef hppDynamicObstacleKCD_H
#define hppDynamicObstacleKCD_H

#include "hppDynamicObstacle/hppDynamicObstacle.h"

#include "kcd2/kcdObject.h"

KIT_PREDEF_CLASS ( ChppDynamicObstacleKCD );

//#define DEBUG_DO_KCD(x)
#define DEBUG_DO_KCD(x) std::cerr << "ChppDynamicObstacleKCD :" << x << std::endl


/**
  \brief  class ChppDynamicObstacleKCD is derived from ChppDynamicObstacle. ChppDynamicEnvHandler modifies a roadmap based on the entering or displacing a ChppDynamcObstacleKCD in the environment. In case of displacing a ChppDynamcObstacleKCD, user shoud call ChppDynamicObstacle::updateFlag(bool inFlag) and set it to true.
 */

class ChppDynamicObstacleKCD : public ChppDynamicObstacle
{
	public:

		virtual ~ChppDynamicObstacleKCD();

		/** \brief Create  */
		static ChppDynamicObstacleKCDShPtr create ( CkcdObjectShPtr inKCDObject,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief Re-implementation of computeModifiedWorkSpaceCells  */
		ktStatus computeModifiedWorkSpaceCells();

		/** \brief Reimplementation of getModifiedObstacle()  */
		CkcdObjectShPtr getModifiedObstacle();

		/** \brief These methods affect the position and transformation of objects in the scene.  */
		ktStatus setAbsolutePosition ( const CkitMat4 &i_matrix );


	protected:

		/** \brief Constructor. */
		ChppDynamicObstacleKCD ( CkcdObjectShPtr inKCDObject, ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief initialization: store weak pointer to itself */
		ktStatus init ( const ChppDynamicObstacleKCDWkPtr &inWeakPtr );

		/** \brief Return the AABB of the attKCDObject */
		ktStatus axisAlignedBoundingBox( double& xMin, double& yMin, double& zMin, double& xMax, double& yMax, double& zMax) const;

		/** \brief Return the angles and the displacement.  */
		void matrixToConfig ( const CkitMat4& i_matrix, std::vector< double >& o_dofValues );


	private:

		/** \brief Store weak pointer to object.  */
		ChppDynamicObstacleKCDWkPtr attWeakPtr;

		/** \brief A pointer to KCDObject */
		CkcdObjectWkPtr attKCDObject;

		/**
		   \if 0
		   \brief Count the number of object created and not destroyed
		*/
		static unsigned int nbObject;
};
#endif
