/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef hppDynamicObstacle_OccupancyMap_H
#define hppDynamicObstacle_OccupancyMap_H

#include "hppDynamicObstacle/hppDynamicObstacle.h"
#include "hppDynamicObstacle/hppPolyhedronMap.h"


KIT_PREDEF_CLASS ( ChppDynamicObstacleOccupancyMap );




/**
  \brief  class ChppDynamicObstacleOccupancyMap is derived from ChppDynamicObstacle. ChppDynamicObstacleOccupancyMap should be  created by using a hppPolyhedronMap. ChppDynamicEnvHandler modifies attRoadmap based on the modification in the hppPolyhedronMap.
 */

class ChppDynamicObstacleOccupancyMap : public ChppDynamicObstacle
{
	public:

		virtual ~ChppDynamicObstacleOccupancyMap();

		/** \brief Create  */
		static ChppDynamicObstacleOccupancyMapShPtr create ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief Re-implementation of computeModifiedWorkSpaceCells  */
		ktStatus computeModifiedWorkSpaceCells();

		/** \brief Update the map.  */
		virtual ktStatus updateEnvironmentKppPolyhedron ( const std::vector<char> inRangeMap );

		/** \brief Reimplementation of getModifiedObstacle()  */
		CkcdObjectShPtr getModifiedObstacle();


	protected:

		/** \brief Constructor. */
		ChppDynamicObstacleOccupancyMap ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief initialization: store weak pointer to itself */
		ktStatus init ( const ChppDynamicObstacleOccupancyMapWkPtr &inWeakPtr );

		std::vector<double> voxel2Pos ( int inVoxelNb );

		/** \brief A set which include the center of voxel which are added in the map*/
		std::vector<point3D> attUpdateReport;

		/** \brief A vector which include the status of each voxel in grid map. each element would be : Free =0, Solid(Obstacle) =1, Unknown =2. Note : In each update of the environment, we use attRangeMap to deattached the last assembely and we use the inRangeMap to the new assembly.*/
		std::vector<char> attRangeMap;

		/** \brief  Number of voxel in teh environment*/
		int attNbOfVoxel;

		/** \brief A pointer to KCDObject */
		ChppPolyhedronMapWkPtr attPolyhedronMap;

		/** \brief A pointer to modified part of environment */

		ChppPolyhedronMapShPtr attModifiedObstacle;

		/** \brief Store weak pointer to object.  */
		ChppDynamicObstacleOccupancyMapWkPtr attWeakPtr;

	private:

		/**
		   \if 0
		   \brief Count the number of object created and not destroyed
		*/
		static unsigned int nbObject;
};
#endif
