/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef hppDynamicObstacle_OccupancyMap_VRML_H
#define hppDynamicObstacle_OccupancyMap_VRML_H

#include "hppDynamicObstacle/hppDynamicObstacleOccupancyMap.h"
#include "hppDynamicObstacle/hppPolyhedronMap.h"


KIT_PREDEF_CLASS ( ChppDynamicObstacleOccupancyMapVRML );

//#define DEBUG_DO_OM_VRML(x)
#define DEBUG_DO_OM_VRML(x) std::cerr << "ChppDynamicObstacleOccupancyMapVRML :" << x << std::endl


/**
  \brief  class ChppDynamicObstacleOccupancyMapVRML is derived from ChppDynamicObstacleOccupancyMap. ChppDynamicObstacleOccupancyMapVRML should be  created by using a hppPolyhedronMap. ChppDynamicEnvHandler modifies attRoadmap based on the modification in the hppPolyhedronMap.
 */

class ChppDynamicObstacleOccupancyMapVRML : public ChppDynamicObstacleOccupancyMap
{
	public:

		virtual ~ChppDynamicObstacleOccupancyMapVRML();

		/** \brief Create  */
		static ChppDynamicObstacleOccupancyMapVRMLShPtr create ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief Update the map. */
		ktStatus updateEnvironmentKppPolyhedron ( const std::vector<char>& inRangeMap );


	protected:

		/** \brief Constructor. */
		ChppDynamicObstacleOccupancyMapVRML ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler );

		/** \brief initialization: store weak pointer to itself */
		ktStatus init ( const ChppDynamicObstacleOccupancyMapVRMLWkPtr &inWeakPtr );


	private:

		/**
		   \if 0
		   \brief Count the number of object created and not destroyed
		*/
		static unsigned int nbObject;
};
#endif
