/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/



/*****************************************
 INCLUDES
*******************************************/

#include <iostream>
#include "hppDynamicObstacle/hppCell.h"

// Select verbosity at configuration by setting CXXFLAGS="... -DDEBUG=[1 or 2]"
#if DEBUG==2
#define ODEBUG2(x) std::cout << "hppCell:" << x << std::endl
#define ODEBUG1(x) std::cerr << "hppCell:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "hppCell:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif



unsigned int ChppCell::nbObject = 0;

// ==========================================================================
ChppCell::ChppCell()
{
	nbObject++;
	ODEBUG2 ( "Constructor of ChppCell. number of ChppCell objects"<< nbObject );
}
// ==========================================================================
ChppCell::~ChppCell()
{
	nbObject--;
	ODEBUG2 ( "-------------------------------------------------------------------------" );
	ODEBUG2 ( "      Destructor of class ChppCell. number of ChppCell objects:" << nbObject );
	ODEBUG2 ( "------------------------------------------------------------------------" );
}
// ==========================================================================
ChppCellShPtr ChppCell::create()
{
	ChppCell*  flatPtr = new ChppCell();
	ChppCellShPtr flatShPtr ( flatPtr );
	ChppCellWkPtr flatWkPtr ( flatShPtr );

	if ( flatPtr->init ( flatWkPtr ) != KD_OK ) flatShPtr.reset();

	return flatShPtr;
}
// ==========================================================================
ktStatus ChppCell::init ( const ChppCellWkPtr &inWeakPtr )
{
	attWeakPtr = inWeakPtr;

	return KD_OK;
}
// ==========================================================================
ktStatus ChppCell::clearCell ()
{
	ODEBUG2 ( "Clearing Cell ...." );
	attUpdateFlag=false;
	attNodeList.clear();
	attEdgeList.clear();
	ODEBUG2 ( "....Cell was cleared" );
	return KD_OK;
}
// ==========================================================================
ktStatus ChppCell::updateFlag ( bool inUpdateFlag )
{
	ODEBUG2 ( "Updating Cell flag ...." );
	attUpdateFlag=inUpdateFlag;
	ODEBUG2 ( "....Flag was updated." );

	return KD_OK;
}
// ==========================================================================
bool ChppCell::updateFlag ()
{
	return attUpdateFlag;
}
// ==========================================================================
ktStatus ChppCell::addNode ( CkwsNodeShPtr inNode )
{
	attNodeList.insert( inNode );
	return KD_OK;
}
// ==========================================================================
ktStatus ChppCell::addEdge ( CkwsEdgeShPtr inEdge )
{
	attEdgeList.insert ( inEdge );

	return KD_OK;
}
// ==========================================================================
std::set<CkwsNodeShPtr> ChppCell::nodeList()
{

	return attNodeList;
}
// ==========================================================================
std::set<CkwsEdgeShPtr> ChppCell::edgeList()
{
	return attEdgeList;
}

