/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

/*****************************************
 INCLUDES
*******************************************/

#include <iostream>
#include "hppDynamicObstacle/hppDynamicObstacleOccupancyMap.h"
#include "hppDynamicObstacle/hppDynamicEnvHandler.h"

// Select verbosity at configuration by setting CXXFLAGS="... -DDEBUG=[1 or 2]"
#if DEBUG==2
#define ODEBUG2(x) std::cout << "hppDynamicObstacleOccupancyMap:" << x << std::endl
#define ODEBUG1(x) std::cerr << "hppDynamicObstacleOccupancyMap:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "hppDynamicObstacleOccupancyMap:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

unsigned int ChppDynamicObstacleOccupancyMap::nbObject = 0;

// ==========================================================================
ChppDynamicObstacleOccupancyMap::ChppDynamicObstacleOccupancyMap ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler ) :ChppDynamicObstacle ( inDynamicEnvHandler )
{
	attPolyhedronMap =inPolyhedronMap;
//-----------
	volume3D att3DMapArea= inDynamicEnvHandler->envGeometry();
	int attMapXN = ( int ) rint ( ( att3DMapArea.xend-att3DMapArea.xstart ) /attPolyhedronMap.lock()->getSize() );
	int attMapYN = ( int ) rint ( ( att3DMapArea.yend-att3DMapArea.ystart ) /attPolyhedronMap.lock()->getSize() );
	int attMapZN = ( int ) rint ( ( att3DMapArea.zend-att3DMapArea.zstart ) /attPolyhedronMap.lock()->getSize() );

	attNbOfVoxel = attMapXN*attMapYN*attMapZN;
	attRangeMap.resize ( attNbOfVoxel );

	for ( int i=0; i < attNbOfVoxel; i++ )
		attRangeMap [i] =0;
//-----------
	nbObject++;
	ODEBUG2 ( "Constructor of ChppDynamicObstacleOccupancyMap. number of objects"<< nbObject );
}
// ==========================================================================
ChppDynamicObstacleOccupancyMap::~ChppDynamicObstacleOccupancyMap()
{
	nbObject--;
	ODEBUG2 ( "---------------------------------------------------------------------------------------------------" );
	ODEBUG2 ( "      Destructor of class ChppDynamicObstacleOccupancyMap. number of objects:" << nbObject );
	ODEBUG2 ( "--------------------------------------------------------------------------------------------------" );
}
// ==========================================================================
ChppDynamicObstacleOccupancyMapShPtr ChppDynamicObstacleOccupancyMap::create ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler )
{
	ChppDynamicObstacleOccupancyMap* flatPtr = new ChppDynamicObstacleOccupancyMap ( inPolyhedronMap, inDynamicEnvHandler );
	ChppDynamicObstacleOccupancyMapShPtr flatShPtr ( flatPtr );
	ChppDynamicObstacleOccupancyMapWkPtr flatWkPtr ( flatShPtr );

	if ( flatPtr->init ( flatWkPtr ) != KD_OK ) flatShPtr.reset();

	return flatShPtr;
}
// ==========================================================================
ktStatus ChppDynamicObstacleOccupancyMap::init ( const ChppDynamicObstacleOccupancyMapWkPtr &inWeakPtr )
{
	ktStatus success = this->ChppDynamicObstacle::init ( inWeakPtr ) ;
	attWeakPtr = inWeakPtr;

	return success;
}
// ==========================================================================
ktStatus ChppDynamicObstacleOccupancyMap::computeModifiedWorkSpaceCells()
{
	ODEBUG2 ( "compute Modified WorkSpace Cells .... " );
	std::vector<double> pos;
	pos.resize ( 3 );

	double size= ( attPolyhedronMap.lock()->getSize() ) /2;
	ODEBUG2 ( "polyhedronMapSize/2 = " << size );

	int updateReportSize = attUpdateReport.size();
	ODEBUG2 ( "updateReportSize = " << updateReportSize );

	for ( int i=0; i<updateReportSize;i++ )
	{
		point3D  pos = attUpdateReport.at ( i );
		getDynamicEnvHandler().lock()->posToCell ( pos.x + size, pos.y + size, pos.z + size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x + size, pos.y + size, pos.z - size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x + size, pos.y - size, pos.z + size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x + size, pos.y - size, pos.z - size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x - size, pos.y + size, pos.z + size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x - size, pos.y + size, pos.z - size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x - size, pos.y - size, pos.z + size )->updateFlag ( true );
		getDynamicEnvHandler ().lock()->posToCell ( pos.x - size, pos.y - size, pos.z - size )->updateFlag ( true );
	}

	ODEBUG2 ( ".....compute Modified WorkSpace Cells was finished. " );

	return KD_OK;
}
// ==========================================================================
ktStatus ChppDynamicObstacleOccupancyMap::updateEnvironmentKppPolyhedron ( const std::vector<char> inRangeMap )
{
	ODEBUG2 ( "updateEnvironmentKppPolyhedron .... " );
	attUpdateReport.clear();
	std::vector<double> pos;
	pos.resize ( 3 );
	point3D  position;
	attPolyhedronMap.lock()->getPolyhedronMap()->removeAllCollisionEntities();

#if 0 // old version which is not compatible with nextbest view algorithm. It seems that there is a conflict with remove child.
// remove grid
	for ( int i=0; i < attNbOfVoxel; i++ )
	{
		if ( attRangeMap.at ( i ) ==1 && inRangeMap.at ( i ) !=1 )
		{
			pos=voxel2Pos ( i );
			attPolyhedronMap.lock()->removePolyhedron ( pos.at ( 0 ), pos.at ( 1 ), pos.at ( 2 ) );
		}
	}

// add new grid
	for ( int i=0; i < attNbOfVoxel; i++ )
	{
		if ( attRangeMap.at ( i ) !=1 && inRangeMap.at ( i ) ==1 )
		{
			pos=voxel2Pos ( i );
			attPolyhedronMap.lock()->addPolyhedron ( pos.at ( 0 ), pos.at ( 1 ), pos.at ( 2 ) );
			position.x=pos.at ( 0 );
			position.y=pos.at ( 1 );
			position.z=pos.at ( 2 );
			attUpdateReport.push_back ( position );
		}
	}
#endif
#if 1 // new version which is not efficient, but compatible with nextbestview
	attPolyhedronMap.lock()->clear();
	for ( int i=0; i < attNbOfVoxel; i++ )
	{
		if ( inRangeMap.at ( i ) ==1 )
		{
			pos=voxel2Pos ( i );
			attPolyhedronMap.lock()->addPolyhedron ( pos.at ( 0 ), pos.at ( 1 ), pos.at ( 2 ) );
			position.x=pos.at ( 0 );
			position.y=pos.at ( 1 );
			position.z=pos.at ( 2 );
			attUpdateReport.push_back ( position );
		}
	}
#endif

	attPolyhedronMap.lock()->getPolyhedronMap()->makeCollisionEntity();

	attRangeMap= inRangeMap;
	if ( attUpdateReport.size() >0 )
	{
		obstacleIsModified();
	}
	ODEBUG2 ( "....... updateEnvironmentKppPolyhedron." );

	return KD_OK;
}
// ============================================================================================
std::vector<double> ChppDynamicObstacleOccupancyMap::voxel2Pos ( int inVoxelNb )
{
	std::vector<double> pos;
	pos.resize ( 3 );
	volume3D att3DMapArea=getDynamicEnvHandler().lock()->envGeometry();
	double attResolution=attPolyhedronMap.lock()->getSize();

	int attMapXN = ( int ) rint ( ( att3DMapArea.xend-att3DMapArea.xstart ) /attResolution );
	int attMapYN = ( int ) rint ( ( att3DMapArea.yend-att3DMapArea.ystart ) /attResolution );

	int NbOfVoxelInLine=attMapXN;
	int NbOfVoxelInLayer=attMapXN*attMapYN;
	int j=0;
	int Nb, lineNb, layerNb;
	double xx,yy,zz;
	j++;

	Nb = inVoxelNb+1;
	lineNb=1;
	layerNb=1;
	if ( Nb> NbOfVoxelInLayer )
	{
		layerNb = ( Nb/NbOfVoxelInLayer ) +1;
		Nb=Nb%NbOfVoxelInLayer;
		if ( Nb==0 )
		{
			layerNb=layerNb-1;
			Nb=NbOfVoxelInLayer;
		}
	}

	if ( Nb> NbOfVoxelInLine )
	{
		lineNb = ( Nb/NbOfVoxelInLine ) +1;
		Nb=Nb%NbOfVoxelInLine;
		if ( Nb==0 )
		{
			lineNb=lineNb-1;
			Nb=NbOfVoxelInLine;
		}
	}
	xx=att3DMapArea.xstart+Nb*attResolution-attResolution/2;
	yy=att3DMapArea.ystart+lineNb*attResolution-attResolution/2;
	zz=att3DMapArea.zstart+layerNb*attResolution-attResolution/2;

	pos [ 0 ] =xx;
	pos [ 1 ] =yy;
	pos [ 2 ] =zz;
	return pos;
}
// ============================================================================================
CkcdObjectShPtr ChppDynamicObstacleOccupancyMap::getModifiedObstacle()
{
	attModifiedObstacle= ChppPolyhedronMap::create ( "modifiedObstacle" , attPolyhedronMap.lock()->getSize() );
	point3D  pos;
	ODEBUG2 ( "size of attUpdateReport = " << attUpdateReport.size() );
	for ( unsigned int i=0; i<attUpdateReport.size();i++ )
	{
		pos=attUpdateReport.at ( i );
		attModifiedObstacle->addPolyhedron ( pos.x, pos.y, pos.z );
	}
	attModifiedObstacle->getPolyhedronMap()->makeCollisionEntity();

	attUpdateReport.clear();

	return attModifiedObstacle->getPolyhedronMap();
}
