/*
   Copyright (c) CNRS,
   @author : Alireza NAKHAEI   anakhaei@laas.fr

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
modification,
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
notice,
   this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
   * Neither the name of the <ORGANIZATION> nor the names of its
contributors
   may be used to endorse or promote products derived from this software
without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

/*****************************************
 INCLUDES
*******************************************/

#include <iostream>
#include "hppDynamicObstacle/hppDynamicObstacleOccupancyMap.h"
#include "hppDynamicObstacle/hppDynamicObstacleOccupancyMapVRML.h"
#include "hppDynamicObstacle/hppDynamicEnvHandler.h"

// Select verbosity at configuration by setting CXXFLAGS="... -DDEBUG=[1 or 2]"
#if DEBUG==2
#define ODEBUG2(x) std::cout << "hppDynamicObstacleOccupancyMapVRML:" << x << std::endl
#define ODEBUG1(x) std::cerr << "hppDynamicObstacleOccupancyMapVRML:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "hppDynamicObstacleOccupancyMapVRML:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif


unsigned int ChppDynamicObstacleOccupancyMapVRML::nbObject = 0;

// ==========================================================================
ChppDynamicObstacleOccupancyMapVRML::ChppDynamicObstacleOccupancyMapVRML( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler ):ChppDynamicObstacleOccupancyMap( inPolyhedronMap, inDynamicEnvHandler )
{
	nbObject++;
	ODEBUG2 ( "Constructor of ChppDynamicObstacleOccupancyMapVRML. number of objects"<< nbObject );
}
// ==========================================================================
ChppDynamicObstacleOccupancyMapVRML::~ChppDynamicObstacleOccupancyMapVRML()
{
	nbObject--;
	ODEBUG2 ( "---------------------------------------------------------------------------------------------------" );
	ODEBUG2 ( "      Destructor of class ChppDynamicObstacleOccupancyMapVRML. number of objects:" << nbObject );
	ODEBUG2 ( "--------------------------------------------------------------------------------------------------" );
}
// ==========================================================================
ChppDynamicObstacleOccupancyMapVRMLShPtr ChppDynamicObstacleOccupancyMapVRML::create ( ChppPolyhedronMapShPtr inPolyhedronMap,ChppDynamicEnvHandlerShPtr inDynamicEnvHandler )
{
	ChppDynamicObstacleOccupancyMapVRML* flatPtr = new ChppDynamicObstacleOccupancyMapVRML ( inPolyhedronMap, inDynamicEnvHandler );
	ChppDynamicObstacleOccupancyMapVRMLShPtr flatShPtr ( flatPtr );
	ChppDynamicObstacleOccupancyMapVRMLWkPtr flatWkPtr ( flatShPtr );

	if ( flatPtr->init ( flatWkPtr ) != KD_OK ) flatShPtr.reset();

	return flatShPtr;
}
// ==========================================================================
ktStatus ChppDynamicObstacleOccupancyMapVRML::init ( const ChppDynamicObstacleOccupancyMapVRMLWkPtr &inWeakPtr )
{
	ktStatus success = this->ChppDynamicObstacleOccupancyMap::init ( inWeakPtr ) ;
	attWeakPtr = inWeakPtr;

	return success;
}
// ==========================================================================
ktStatus ChppDynamicObstacleOccupancyMapVRML::updateEnvironmentKppPolyhedron ( const std::vector<char>& inRangeMap )
{
	ODEBUG2 ( "updateEnvironmentKppPolyhedron .... " );
	attUpdateReport.clear();
	std::vector<double> pos;
	pos.resize ( 3 );
	point3D  position;

	ODEBUG2 ( "attNbOfVoxel = "  << attNbOfVoxel );

// add new grid
	for ( int i=0; i < attNbOfVoxel; i++ )
	{
		if ( attRangeMap.at ( i ) !=1 && inRangeMap.at ( i ) ==1 )
		{
			pos=voxel2Pos ( i );
			position.x=pos.at ( 0 );
			position.y=pos.at ( 1 );
			position.z=pos.at ( 2 );
			attUpdateReport.push_back ( position );
		}
	}
	attRangeMap= inRangeMap;
	obstacleIsModified ();
	ODEBUG2 ( "....... updateEnvironmentKppPolyhedron." );

	return KD_OK;
}

